@@ -32,3 +32,6 @@ from .decompiler import Decompiler
 from .soot_class_hierarchy import SootClassHierarchy
 from .propagator import PropagatorAnalysis
 from .xrefs import XRefsAnalysis
+from .init_finder import InitializationFinder
+from .complete_calling_conventions import CompleteCallingConventionsAnalysis
+from .typehoon import Typehoon
