@@ -4,3 +4,5 @@ from .structured_codegen import StructuredCodeGenerator
 from .clinic import Clinic
 from .region_simplifier import RegionSimplifier
 from .decompiler import Decompiler
+from .decompilation_options import options, options_by_category
+from . import optimization_passes
