@@ -2,7 +2,10 @@ import logging
 
 import claripy
 import archinfo
+from archinfo import RegisterName
+from typing import Union, Optional, List, Dict
 
+from .sim_type import SimType
 from .sim_type import SimTypeChar
 from .sim_type import SimTypePointer
 from .sim_type import SimTypeFixedSizeArray
@@ -91,7 +94,7 @@ class SimRegArg(SimFunctionArgument):
     :ivar string reg_name:    The name of the represented register.
     :ivar int size:           The size of the register, in number of bytes.
     """
-    def __init__(self, reg_name, size, alt_offsets=None):
+    def __init__(self, reg_name: RegisterName, size: int, alt_offsets=None):
         SimFunctionArgument.__init__(self, size)
         self.reg_name = reg_name
         self.alt_offsets = {} if alt_offsets is None else alt_offsets
@@ -182,7 +185,7 @@ class SimComboArg(SimFunctionArgument):
         return type(other) is SimComboArg and all(a == b for a, b in zip(self.locations, other.locations))
 
     def set_value(self, state, value, endness=None, **kwargs):  # pylint:disable=arguments-differ
-        # TODO: This code needs to be reworked for variable byte with and the Third Endness
+        # TODO: This code needs to be reworked for variable byte width and the Third Endness
         self.check_value(value)
         if endness is None: endness = state.arch.memory_endness
         if isinstance(value, int):
@@ -192,6 +195,7 @@ class SimComboArg(SimFunctionArgument):
                 raise ValueError("What do I do with a float %d bytes long" % self.size)
             value = claripy.FPV(value, claripy.FSORT_FLOAT if self.size == 4 else claripy.FSORT_DOUBLE)
         cur = 0
+        # TODO: I have no idea if this reversed is only supposed to be applied in LE situations
         for loc in reversed(self.locations):
             loc.set_value(state, value[cur*state.arch.byte_width + loc.size*state.arch.byte_width - 1:cur*state.arch.byte_width], endness=endness, **kwargs)
             cur += loc.size
@@ -199,15 +203,18 @@ class SimComboArg(SimFunctionArgument):
     def get_value(self, state, endness=None, **kwargs):  # pylint:disable=arguments-differ
         if endness is None: endness = state.arch.memory_endness
         vals = []
-        for loc in self.locations:
+        for loc in reversed(self.locations):
             vals.append(loc.get_value(state, endness, **kwargs))
-        return claripy.Concat(*vals)
+        return state.solver.Concat(*vals)
 
 
 class ArgSession:
     """
     A class to keep track of the state accumulated in laying parameters out into memory
     """
+
+    __slots__ = ('cc', 'real_args', 'fp_iter', 'int_iter', 'both_iter', )
+
     def __init__(self, cc):
         self.cc = cc
         self.real_args = None
@@ -215,16 +222,18 @@ class ArgSession:
         self.int_iter = None
         self.both_iter = None
 
-        if cc.args is None:
-            self.fp_iter = cc.fp_args
-            self.int_iter = cc.int_args
-            self.both_iter = cc.both_args
-        else:
+        # these iters should only be used if real_args are not set or real_args are intentionally ignored (e.g., when
+        # variadic arguments are used).
+        self.fp_iter = cc.fp_args
+        self.int_iter = cc.int_args
+        self.both_iter = cc.both_args
+
+        if cc.args is not None:
             self.real_args = iter(cc.args)
 
     # TODO: use safer errors than TypeError and ValueError
-    def next_arg(self, is_fp, size=None):
-        if self.real_args is not None:
+    def next_arg(self, is_fp, size=None, ignore_real_args=False):
+        if self.real_args is not None and not ignore_real_args:
             try:
                 arg = next(self.real_args)
                 if is_fp and self.cc.is_fp_arg(arg) is False:
@@ -276,7 +285,12 @@ class SimCC:
 
     An instance of this class allows it to be tweaked to the way a specific function should be called.
     """
-    def __init__(self, arch, args=None, ret_val=None, sp_delta=None, func_ty=None):
+    def __init__(self,
+            arch: archinfo.Arch,
+            args: Optional[List[SimFunctionArgument]]=None,
+            ret_val: Optional[SimFunctionArgument]=None,
+            sp_delta: Optional[int]=None,
+            func_ty: Optional[Union[SimTypeFunction, str]]=None):
         """
         :param arch:        The Archinfo arch for this CC
         :param args:        A list of SimFunctionArguments describing where the arguments go
@@ -309,7 +323,7 @@ class SimCC:
         self.args = args
         self.ret_val = ret_val
         self.sp_delta = sp_delta
-        self.func_ty = func_ty if func_ty is None else func_ty.with_arch(arch)
+        self.func_ty: Optional[SimTypeFunction] = func_ty if func_ty is None else func_ty.with_arch(arch)
 
     @classmethod
     def from_arg_kinds(cls, arch, fp_args, ret_fp=False, sizes=None, sp_delta=None, func_ty=None):
@@ -334,17 +348,18 @@ class SimCC:
     # Here are all the things a subclass needs to specify!
     #
 
-    ARG_REGS = None                 # A list of all the registers used for integral args, in order (names or offsets)
-    FP_ARG_REGS = None              # A list of all the registers used for floating point args, in order
-    STACKARG_SP_BUFF = 0            # The amount of stack space reserved between the saved return address
-                                    # (if applicable) and the arguments. Probably zero.
-    STACKARG_SP_DIFF = 0            # The amount of stack space reserved for the return address
-    CALLER_SAVED_REGS = None        # Caller-saved registers
-    RETURN_ADDR = None              # The location where the return address is stored, as a SimFunctionArgument
-    RETURN_VAL = None               # The location where the return value is stored, as a SimFunctionArgument
-    FP_RETURN_VAL = None            # The location where floating-point argument return values are stored
-    ARCH = None                     # The archinfo.Arch class that this CC must be used for, if relevant
-    CALLEE_CLEANUP = False          # Whether the callee has to deallocate the stack space for the arguments
+    ARG_REGS: List[str] = None                                  # A list of all the registers used for integral args, in order (names or offsets)
+    FP_ARG_REGS: List[str] = None                               # A list of all the registers used for floating point args, in order
+    STACKARG_SP_BUFF = 0                                        # The amount of stack space reserved between the saved return address
+                                                                # (if applicable) and the arguments. Probably zero.
+    STACKARG_SP_DIFF = 0                                        # The amount of stack space reserved for the return address
+    CALLER_SAVED_REGS: List[str] = None                         # Caller-saved registers
+    RETURN_ADDR: SimFunctionArgument = None                     # The location where the return address is stored, as a SimFunctionArgument
+    RETURN_VAL: SimFunctionArgument = None                      # The location where the return value is stored, as a SimFunctionArgument
+    OVERFLOW_RETURN_VAL: Optional[SimFunctionArgument] = None   # The second half of the location where a double-length return value is stored
+    FP_RETURN_VAL: Optional[SimFunctionArgument] = None         # The location where floating-point argument return values are stored
+    ARCH = None                                                 # The archinfo.Arch class that this CC must be used for, if relevant
+    CALLEE_CLEANUP = False                                      # Whether the callee has to deallocate the stack space for the arguments
 
     STACK_ALIGNMENT = 1             # the alignment requirement of the stack pointer at function start BEFORE call
 
@@ -440,7 +455,16 @@ class SimCC:
         The location the return value is stored.
         """
         # pylint: disable=unsubscriptable-object
-        return self.RETURN_VAL if self.ret_val is None else self.ret_val
+        if self.ret_val is not None:
+            return self.ret_val
+
+        if self.func_ty is not None and \
+                self.func_ty.returnty is not None and \
+                self.OVERFLOW_RETURN_VAL is not None and \
+                self.func_ty.returnty.size not in (None, NotImplemented) and \
+                self.func_ty.returnty.size > self.RETURN_VAL.size * self.arch.byte_width:
+            return SimComboArg([self.RETURN_VAL, self.OVERFLOW_RETURN_VAL])
+        return self.RETURN_VAL
 
     @property
     def fp_return_val(self):
@@ -471,13 +495,32 @@ class SimCC:
         If you've customized this CC, this will sanity-check the provided locations with the given list.
         """
         session = self.arg_session
-        if self.func_ty is None:
-            # No function prototype is provided. `is_fp` must be provided.
+        if self.func_ty is None and self.args is None:
+            # No function prototype is provided, no args is provided. `is_fp` must be provided.
             if is_fp is None:
                 raise ValueError('"is_fp" must be provided when no function prototype is available.')
         else:
-            # let's rely on the func_ty for the number of arguments and whether each argument is FP or not
-            is_fp = [ True if isinstance(arg, (SimTypeFloat, SimTypeDouble)) else False for arg in self.func_ty.args ]
+            # let's rely on the func_ty or self.args for the number of arguments and whether each argument is FP or not
+            if self.func_ty is not None:
+                args = [ a.with_arch(self.arch) for a in self.func_ty.args ]
+            else:
+                args = self.args
+            is_fp = [ True if isinstance(arg, (SimTypeFloat, SimTypeDouble)) or self.is_fp_arg(arg) else False
+                      for arg in args ]
+            if sizes is None:
+                # initialize sizes from args
+                sizes = [ ]
+                for a in args:
+                    if isinstance(a, SimType):
+                        if isinstance(a, SimTypeFixedSizeArray) or a.size is NotImplemented:
+                            sizes.append(self.arch.bytes)
+                        else:
+                            sizes.append(a.size // 8)  # SimType.size is in bits
+                    elif isinstance(a, SimFunctionArgument):
+                        sizes.append(a.size)  # SimFunctionArgument.size is in bytes
+                    else:
+                        # fallback to use self.arch.bytes
+                        sizes.append(self.arch.bytes)
 
         if sizes is None: sizes = [self.arch.bytes] * len(is_fp)
         return [session.next_arg(ifp, size=sz) for ifp, sz in zip(is_fp, sizes)]
@@ -493,8 +536,10 @@ class SimCC:
         you've customized this CC.
         """
         session = self.arg_session
-        if self.args is None:
-            arg_loc = [session.next_arg(False) for _ in range(index + 1)][-1]
+        if self.args is None or index >= len(self.args):
+            # self.args may not be provided, or args is incorrect or includes variadic arguments that we must get the
+            # proper argument according to the default calling convention
+            arg_loc = [session.next_arg(False, ignore_real_args=True) for _ in range(index + 1)][-1]
         else:
             arg_loc = self.args[index]
 
@@ -717,11 +762,11 @@ class SimCC:
         if self.ret_val is not None:
             loc = self.ret_val
         elif is_fp is not None:
-            loc = self.FP_RETURN_VAL if is_fp else self.RETURN_VAL
+            loc = self.fp_return_val if is_fp else self.return_val
         elif ty is not None:
-            loc = self.FP_RETURN_VAL if isinstance(ty, SimTypeFloat) else self.RETURN_VAL
+            loc = self.fp_return_val if isinstance(ty, SimTypeFloat) else self.return_val
         else:
-            loc = self.FP_RETURN_VAL if self.is_fp_value(val) else self.RETURN_VAL
+            loc = self.fp_return_val if self.is_fp_value(val) else self.return_val
 
         if loc is None:
             raise NotImplementedError("This SimCC doesn't know how to store this value - should be implemented")
@@ -883,7 +928,10 @@ class SimCC:
             raise TypeError("I don't know how to serialize %s." % repr(arg))
 
     def __repr__(self):
-        return "<" + self.__class__.__name__ + '>'
+        return "<{}: {}->{}, sp_delta={}>".format(self.__class__.__name__,
+                                                  self.args,
+                                                  self.ret_val,
+                                                  self.sp_delta)
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -995,6 +1043,7 @@ class SimCCCdecl(SimCC):
     STACKARG_SP_DIFF = 4 # Return address is pushed on to stack by call
     CALLER_SAVED_REGS = ['eax', 'ecx', 'edx']
     RETURN_VAL = SimRegArg('eax', 4)
+    OVERFLOW_RETURN_VAL = SimRegArg('edx', 4)
     FP_RETURN_VAL = SimLyingRegArg('st0')
     RETURN_ADDR = SimStackArg(0, 4)
     ARCH = archinfo.ArchX86
@@ -1008,9 +1057,11 @@ class SimCCMicrosoftAMD64(SimCC):
     STACKARG_SP_DIFF = 8 # Return address is pushed on to stack by call
     STACKARG_SP_BUFF = 32 # 32 bytes of shadow stack space
     RETURN_VAL = SimRegArg('rax', 8)
+    OVERFLOW_RETURN_VAL = SimRegArg('rdx', 8)
     FP_RETURN_VAL = SimRegArg('xmm0', 32)
     RETURN_ADDR = SimStackArg(0, 8)
     ARCH = archinfo.ArchAMD64
+    STACK_ALIGNMENT = 16
 
 class SimCCX86LinuxSyscall(SimCC):
     ARG_REGS = ['ebx', 'ecx', 'edx', 'esi', 'edi', 'ebp']
@@ -1052,6 +1103,7 @@ class SimCCSystemVAMD64(SimCC):
     CALLER_SAVED_REGS = [ 'rdi', 'rsi', 'rdx', 'rcx', 'r8', 'r9', 'r10', 'r11', 'rax', ]
     RETURN_ADDR = SimStackArg(0, 8)
     RETURN_VAL = SimRegArg('rax', 8)
+    OVERFLOW_RETURN_VAL = SimRegArg('rdx', 8)
     FP_RETURN_VAL = SimRegArg('xmm0', 32)
     ARCH = archinfo.ArchAMD64
     STACK_ALIGNMENT = 16
@@ -1119,7 +1171,7 @@ class SimCCAMD64WindowsSyscall(SimCC):
 class SimCCARM(SimCC):
     ARG_REGS = [ 'r0', 'r1', 'r2', 'r3' ]
     FP_ARG_REGS = []    # TODO: ???
-    CALLER_SAVED_REGS = []   # TODO: ???
+    CALLER_SAVED_REGS = []
     RETURN_ADDR = SimRegArg('lr', 4)
     RETURN_VAL = SimRegArg('r0', 4)
     ARCH = archinfo.ArchARM
@@ -1343,7 +1395,7 @@ CC = {
 }
 
 
-DEFAULT_CC = {
+DEFAULT_CC: Dict[str,type] = {
     'AMD64': SimCCSystemVAMD64,
     'X86': SimCCCdecl,
     'ARMEL': SimCCARM,
