@@ -1,14 +1,16 @@
-
 # pylint:disable=no-self-use
+from typing import Tuple
 
 import logging
 
 import ailment
 import pyvex
+import archinfo
 
+from ...engines.vex.claripy.irop import operations as vex_operations
+from ...code_location import CodeLocation
+from ...utils.constants import DEFAULT_STATEMENT
 from ..engine import SimEngine
-from angr.engines.vex.claripy.irop import operations as vex_operations
-from ...analyses.code_location import CodeLocation
 
 
 class SimEngineLight(SimEngine):
@@ -21,6 +23,7 @@ class SimEngineLight(SimEngine):
         self.state = None
         self.arch = None
         self.block = None
+        self._call_stack = None
 
         self.stmt_idx = None
         self.ins_addr = None
@@ -44,10 +47,20 @@ class SimEngineLight(SimEngine):
     # Helper methods
     #
 
+    @property
+    def _context(self) -> Tuple[int]:
+        if not self._call_stack:
+            return tuple()
+
+        # Convert to Tuple to make `context` hashable if not None
+        call_stack_addresses = tuple(self._call_stack)
+        return call_stack_addresses
+
     def _codeloc(self, block_only=False):
         return CodeLocation(self.block.addr,
                             None if block_only else self.stmt_idx,
                             ins_addr=None if block_only else self.ins_addr,
+                            context=self._context
                             )
 
 
@@ -63,7 +76,7 @@ class SimEngineLightVEXMixin:
         self.state = state
 
         if state is not None:
-            self.arch = state.arch
+            self.arch: archinfo.Arch = state.arch
 
         self.tyenv = block.vex.tyenv
 
@@ -90,7 +103,15 @@ class SimEngineLightVEXMixin:
 
             self._handle_Stmt(stmt)
 
+        self._process_block_end()
+
+    def _process_block_end(self):
+        # handle calls to another function
+        # Note that without global information, we cannot handle cases where we *jump* to another function (jumpkind ==
+        # "Ijk_Boring"). Users are supposed to overwrite this method, detect these cases with the help of global
+        # information (such as CFG or symbol addresses), and handle them accordingly.
         if self.block.vex.jumpkind == 'Ijk_Call':
+            self.stmt_idx = DEFAULT_STATEMENT
             handler = '_handle_function'
             if hasattr(self, handler):
                 func_addr = self._expr(self.block.vex.next)
@@ -132,6 +153,12 @@ class SimEngineLightVEXMixin:
     def _handle_Store(self, stmt):
         raise NotImplementedError('Please implement the Store handler with your own logic.')
 
+    def _handle_StoreG(self, stmt):
+        raise NotImplementedError('Please implement the StoreG handler with your own logic.')
+
+    def _handle_LLSC(self, stmt: pyvex.IRStmt.LLSC):
+        raise NotImplementedError('Please implement the LLSC handler with your own logic.')
+
     #
     # Expression handlers
     #
@@ -158,6 +185,9 @@ class SimEngineLightVEXMixin:
     def _handle_Load(self, expr):
         raise NotImplementedError('Please implement the Load handler with your own logic.')
 
+    def _handle_LoadG(self, stmt):
+        raise NotImplementedError('Please implement the LoadG handler with your own logic.')
+
     def _handle_Exit(self, stmt):
         self._expr(stmt.guard)
         self._expr(stmt.dst)
@@ -177,7 +207,7 @@ class SimEngineLightVEXMixin:
 
         # All conversions are handled by the Conversion handler
         simop = vex_operations.get(expr.op)
-        if simop is not None and simop.op_attrs['conversion']:
+        if simop is not None and simop.op_attrs.get('conversion', None):
             handler = '_handle_Conversion'
         # Notice order of "Not" comparisons
         elif expr.op == 'Iop_Not1':
@@ -549,7 +579,7 @@ class SimEngineLightAILMixin:
             self.stmt_idx = stmt_idx
             self.ins_addr = stmt.ins_addr
 
-            self._ail_handle_Stmt(stmt)
+            self._handle_Stmt(stmt)
 
     def _expr(self, expr):
 
@@ -564,18 +594,25 @@ class SimEngineLightAILMixin:
     #
 
     def _codeloc(self):
-        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr)
+        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr, context=self._context)
 
     #
     # Statement handlers
     #
 
-    def _ail_handle_Stmt(self, stmt):
-        handler = "_ail_handle_%s" % type(stmt).__name__
+    def _handle_Stmt(self, stmt):
+        handler = "_handle_%s" % type(stmt).__name__
         if hasattr(self, handler):
             getattr(self, handler)(stmt)
-        else:
-            self.l.warning('Unsupported statement type %s.', type(stmt).__name__)
+            return
+
+        # compatibility
+        old_handler = "_ail_handle_%s" % type(stmt).__name__
+        if hasattr(self, old_handler):
+            getattr(self, old_handler)(stmt)
+            return
+
+        self.l.warning('Unsupported statement type %s.', type(stmt).__name__)
 
     def _ail_handle_Jump(self, stmt):
         raise NotImplementedError('Please implement the Jump handler with your own logic.')
@@ -639,7 +676,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 <= expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'CmpLT', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'CmpLT', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Add(self, expr):
 
@@ -655,7 +692,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 + expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Add', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Add', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Sub(self, expr):
 
@@ -672,7 +709,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 - expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Sub', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Sub', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Div(self, expr):
 
@@ -689,7 +726,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 // expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Div', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Div', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_DivMod(self, expr):
         return self._ail_handle_Div(expr)
@@ -709,7 +746,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 * expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Mul', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Mul', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Mull(self, expr):
         return self._ail_handle_Mul(expr)
@@ -729,7 +766,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 & expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'And', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'And', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Or(self, expr):
 
@@ -746,7 +783,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 | expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Or', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Or', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Xor(self, expr):
 
@@ -763,7 +800,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 ^ expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Xor', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Xor', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Shr(self, expr):
 
@@ -779,7 +816,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 >> expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Shr', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Shr', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Shl(self, expr):
 
@@ -795,7 +832,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 << expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Shl', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Shl', [expr_0, expr_1], expr.signed, **expr.tags)
 
     def _ail_handle_Sal(self, expr):
         return self._ail_handle_Shl(expr)
@@ -814,7 +851,7 @@ class SimEngineLightAILMixin:
         try:
             return expr_0 >> expr_1
         except TypeError:
-            return ailment.Expr.BinaryOp(expr.idx, 'Sar', [expr_0, expr_1], **expr.tags)
+            return ailment.Expr.BinaryOp(expr.idx, 'Sar', [expr_0, expr_1], expr.signed, **expr.tags)
 
     #
     # Unary operation handlers
@@ -830,6 +867,8 @@ class SimEngineLightAILMixin:
     def _ail_handle_Not(self, expr):
 
         data = self._expr(expr.operand)
+        if data is None:
+            return None
 
         try:
             return ~data  # pylint:disable=invalid-unary-operand-type
