@@ -116,6 +116,7 @@ class SimSuccessors:
         state.scratch.source = source if source is not None else self.addr
         state.scratch.exit_stmt_idx = exit_stmt_idx
         state.scratch.exit_ins_addr = exit_ins_addr
+        state.history.jump_source = state.scratch.exit_ins_addr
 
         self._preprocess_successor(state, add_guard=add_guard)
 
@@ -184,7 +185,7 @@ class SimSuccessors:
                     ret_addr = state.mem[state.regs._sp].long.concrete
                 else:
                     ret_addr = state.solver.eval(state.regs._lr)
-            except SimSolverModeError:
+            except (SimSolverModeError, SimUnsatError):
                 # Use the address for UnresolvableCallTarget instead.
                 ret_addr = state.project.simos.unresolvable_call_target
 
@@ -195,7 +196,7 @@ class SimSuccessors:
 
             try:
                 stack_ptr = state.solver.eval(state.regs._sp)
-            except SimSolverModeError:
+            except (SimSolverModeError, SimUnsatError):
                 stack_ptr = 0
 
             new_frame = CallStack(
@@ -296,10 +297,12 @@ class SimSuccessors:
             _max_targets = state.options.symbolic_ip_max_targets
             _max_jumptable_targets = state.options.jumptable_symbolic_ip_max_targets
             try:
+                skip_max_targets_warning = False
                 if o.NO_IP_CONCRETIZATION in state.options:
                     # Don't try to concretize the IP
                     cond_and_targets = [ (claripy.true, target) ]
                     max_targets = 0
+                    skip_max_targets_warning = True  # don't warn
                 elif o.KEEP_IP_SYMBOLIC in state.options:
                     s = claripy.Solver()
                     addrs = s.eval(target, _max_targets + 1, extra_constraints=tuple(state.ip_constraints))
@@ -320,11 +323,12 @@ class SimSuccessors:
                         max_targets = _max_jumptable_targets
 
                 if len(cond_and_targets) > max_targets:
-                    l.warning(
-                        "Exit state has over %d possible solutions. Likely unconstrained; skipping. %s",
-                        max_targets,
-                        target.shallow_repr()
-                    )
+                    if not skip_max_targets_warning:
+                        l.warning(
+                            "Exit state has over %d possible solutions. Likely unconstrained; skipping. %s",
+                            max_targets,
+                            target.shallow_repr()
+                        )
                     self.unconstrained_successors.append(state)
                 else:
                     for cond, a in cond_and_targets:
@@ -514,7 +518,7 @@ class SimSuccessors:
 
 
 from ..state_plugins.inspect import BP_BEFORE, BP_AFTER
-from ..errors import SimSolverModeError, AngrUnsupportedSyscallError, AngrSyscallError, SimValueError
+from ..errors import SimSolverModeError, AngrUnsupportedSyscallError, AngrSyscallError, SimValueError, SimUnsatError
 from ..calling_conventions import SYSCALL_CC
 from ..state_plugins.sim_action_object import _raw_ast
 from ..state_plugins.callstack import CallStack
