@@ -1,3 +1,5 @@
+from typing import Dict, Optional
+
 import claripy
 import logging
 from archinfo.arch_arm import is_arm_arch
@@ -35,10 +37,18 @@ def boolean_extend(O, a, b, size):
 def op_concretize(op):
     if type(op) is int:
         return op
+    op = op.ite_excavated
+    if op.op == 'If':
+        cases = list(claripy.reverse_ite_cases(op))
+        if all(c.op == 'BVV' for _, c in cases):
+            raise CCallMultivaluedException(cases)
     if op.op != 'BVV':
         raise SimError("Hit a symbolic conditional operation. Something has gone wildly wrong.")
     return op.args[0]
 
+class CCallMultivaluedException(Exception):
+    pass
+
 ##################
 ### x86* data ###
 ##################
@@ -55,7 +65,7 @@ data = {
         'CondBitMasks': { },
         'OpTypes': { },
     }
-}
+} # type: Dict[str, Dict[str, Dict[str, Optional[int]]]]
 
 # condition types
 data['AMD64']['CondTypes']['CondO']      = 0  # /* overflow           */
@@ -146,6 +156,10 @@ data['AMD64']['OpTypes']['G_CC_OP_SMULW'] = 50
 data['AMD64']['OpTypes']['G_CC_OP_SMULL'] = 51
 data['AMD64']['OpTypes']['G_CC_OP_SMULQ'] = 52
 data['AMD64']['OpTypes']['G_CC_OP_NUMBER'] = 53
+data['AMD64']['OpTypes']['G_CC_OP_ADCXL'] = 61
+data['AMD64']['OpTypes']['G_CC_OP_ADCXQ'] = 62
+data['AMD64']['OpTypes']['G_CC_OP_ADOXL'] = 63
+data['AMD64']['OpTypes']['G_CC_OP_ADOXQ'] = 64
 
 data['X86']['CondTypes']['CondO']      = 0
 data['X86']['CondTypes']['CondNO']     = 1
@@ -328,6 +342,33 @@ def pc_actions_ADC(state, nbits, cc_dep1, cc_dep2, cc_ndep, platform=None):
 
     return pc_make_rdata(data[platform]['size'], cf, pf, af, zf, sf, of, platform=platform)
 
+def pc_actions_ADCX(state, nbits, cc_dep1, cc_dep2, cc_ndep, is_adc, platform=None):
+    pf = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_P'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_P']]
+    af = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_A'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_A']]
+    zf = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_Z'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_Z']]
+    sf = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_S'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_S']]
+    if is_adc:
+        carry = claripy.LShR(cc_ndep, data[platform]['CondBitOffsets']['G_CC_SHIFT_C']) & 1
+        of = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_O'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_O']]
+    else:
+        carry = claripy.LShR(cc_ndep, data[platform]['CondBitOffsets']['G_CC_SHIFT_O']) & 1
+        cf = (cc_ndep & data[platform]['CondBitMasks']['G_CC_MASK_C'])[data[platform]['CondBitOffsets']['G_CC_SHIFT_C']]
+    arg_l = cc_dep1
+    arg_r = cc_dep2 ^ carry
+    res = (arg_l + arg_r) + carry
+
+    carry = claripy.If(
+            carry != 0,
+            claripy.If(res <= arg_l, claripy.BVV(1, 1), claripy.BVV(0, 1)),
+            claripy.If(res < arg_l, claripy.BVV(1, 1), claripy.BVV(0, 1))
+    )
+    if is_adc:
+        cf = carry
+    else:
+        of = carry
+
+    return pc_make_rdata(data[platform]['size'], cf, pf, af, zf, sf, of, platform=platform)
+
 def pc_actions_SBB(state, nbits, cc_dep1, cc_dep2, cc_ndep, platform=None):
     old_c = cc_ndep[data[platform]['CondBitOffsets']['G_CC_SHIFT_C']].zero_extend(nbits-1)
     arg_l = cc_dep1
@@ -501,6 +542,13 @@ def pc_calculate_rdata_all_WRK(state, cc_op, cc_dep1_formal, cc_dep2_formal, cc_
         l.debug("cc_str: SMULQ")
         return pc_actions_SMULQ(state, nbits, cc_dep1_formal, cc_dep2_formal, cc_ndep_formal, platform=platform)
 
+    if cc_str in [ 'G_CC_OP_ADOXL', 'G_CC_OP_ADOXQ' ]:
+        l.debug("cc_str: ADOX")
+        return pc_actions_ADCX(state, nbits, cc_dep1_formal, cc_dep2_formal, cc_ndep_formal, False, platform=platform)
+    if cc_str in [ 'G_CC_OP_ADCXL', 'G_CC_OP_ADCXQ' ]:
+        l.debug("cc_str: ADCX")
+        return pc_actions_ADCX(state, nbits, cc_dep1_formal, cc_dep2_formal, cc_ndep_formal, True, platform=platform)
+
     l.error("Unsupported cc_op %d in in pc_calculate_rdata_all_WRK", cc_op)
     raise SimCCallError("Unsupported cc_op in pc_calculate_rdata_all_WRK")
 
@@ -1446,8 +1494,10 @@ def armg_calculate_flags_nzcv(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
 
 
 def armg_calculate_condition(state, cond_n_op, cc_dep1, cc_dep2, cc_dep3):
-    cond = claripy.LShR(cond_n_op, 4)
-    cc_op = cond_n_op & 0xF
+    concrete_cond_n_op = op_concretize(cond_n_op)
+
+    cond = concrete_cond_n_op >> 4
+    cc_op = concrete_cond_n_op & 0xF
     inv = cond & 1
 
     concrete_cond = op_concretize(cond)
@@ -1564,7 +1614,10 @@ def arm64g_calculate_flag_n(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op == ARM64G_CC_OP_LOGIC64:
         flag = claripy.LShR(cc_dep1, 63)
 
-    if flag is not None: return flag
+    if flag is not None:
+        if len(flag) == 32:
+            flag = flag.zero_extend(32)
+        return flag
     l.error("Unknown cc_op %s (arm64g_calculate_flag_n)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
 
@@ -1614,7 +1667,10 @@ def arm64g_calculate_flag_z(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op in (ARM64G_CC_OP_LOGIC32, ARM64G_CC_OP_LOGIC64):
         flag = arm64_zerobit(state, cc_dep1)
 
-    if flag is not None: return flag
+    if flag is not None:
+        if len(flag) == 32:
+            flag = flag.zero_extend(32)
+        return flag
 
     l.error("Unknown cc_op %s (arm64g_calculate_flag_z)", concrete_op)
     raise SimCCallError("Unknown cc_op %s" % concrete_op)
@@ -1718,11 +1774,12 @@ def arm64g_calculate_data_nzcv(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     return _concat_flags(ARM64G_NBITS, vec)
 
 def arm64g_calculate_condition(state, cond_n_op, cc_dep1, cc_dep2, cc_dep3):
-    cond = claripy.LShR(cond_n_op, 4)
-    cc_op = cond_n_op & 0xF
+    concretize_cond_n_op = op_concretize(cond_n_op)
+    cond = concretize_cond_n_op >> 4
+    cc_op = concretize_cond_n_op & 0xF
     inv = cond & 1
 
-    concrete_cond = op_concretize(cond)
+    concrete_cond = cond
     flag = None
 
     if concrete_cond in (ARM64CondAL, ARM64CondNV):
