@@ -1,17 +1,19 @@
 import logging
+from typing import List, Optional, Union, overload
 from archinfo.arch_soot import ArchSoot, SootAddressDescriptor
 
+
 from .sim_state import SimState
 from .calling_conventions import DEFAULT_CC, SimRegArg, SimStackArg, PointerWrapper
 from .callable import Callable
 from .errors import AngrAssemblyError
-from .engines import UberEngine, ProcedureEngine
+from .engines import UberEngine, ProcedureEngine, SimEngineConcrete
 
 
 l = logging.getLogger(name=__name__)
 
 
-class AngrObjectFactory(object):
+class AngrObjectFactory():
     """
     This factory provides access to important analysis elements.
     """
@@ -24,6 +26,11 @@ class AngrObjectFactory(object):
         self.default_engine = default_engine(project)
         self.procedure_engine = ProcedureEngine(project)
 
+        if project.concrete_target:
+            self.concrete_engine = SimEngineConcrete(project)
+        else:
+            self.concrete_engine = None
+
     def snippet(self, addr, jumpkind=None, **block_opts):
         if self.project.is_hooked(addr) and jumpkind != 'Ijk_NoHook':
             hook = self.project._sim_procedures[addr]
@@ -48,7 +55,6 @@ class AngrObjectFactory(object):
 
         Additional keyword arguments will be passed directly into each engine's process method.
         """
-
         if engine is not None:
             return engine.process(*args, **kwargs)
         return self.default_engine.process(*args, **kwargs)
@@ -70,7 +76,7 @@ class AngrObjectFactory(object):
         """
         return self.project.simos.state_blank(**kwargs)
 
-    def entry_state(self, **kwargs):
+    def entry_state(self, **kwargs) -> SimState:
         """
         Returns a state object representing the program at its entry point. All parameters are optional.
 
@@ -159,7 +165,7 @@ class AngrObjectFactory(object):
         """
         return self.project.simos.state_call(addr, *args, **kwargs)
 
-    def simulation_manager(self, thing=None, **kwargs):
+    def simulation_manager(self, thing: Optional[Union[List[SimState], SimState]]=None, **kwargs) -> 'SimulationManager':
         """
         Constructs a new simulation manager.
 
@@ -270,10 +276,26 @@ class AngrObjectFactory(object):
                 sp_delta=sp_delta,
                 func_ty=func_ty)
 
-    def block(self, addr, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
+    #pylint: disable=unused-argument, no-self-use, function-redefined
+    @overload
+    def block(self, addr: int, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
               extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
               insn_bytes=None, insn_text=None,  # backward compatibility
               strict_block_end=None, collect_data_refs=False,
+              ) -> 'Block': ...
+
+    #pylint: disable=unused-argument, no-self-use, function-redefined
+    @overload
+    def block(self, addr: SootAddressDescriptor, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
+              extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
+              insn_bytes=None, insn_text=None,  # backward compatibility
+              strict_block_end=None, collect_data_refs=False,
+              ) -> 'SootBlock': ...
+
+    def block(self, addr, size=None, max_size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
+              extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
+              insn_bytes=None, insn_text=None,  # backward compatibility
+              strict_block_end=None, collect_data_refs=False, cross_insn_opt=True,
               ):
 
         if isinstance(self.project.arch, ArchSoot) and isinstance(addr, SootAddressDescriptor):
@@ -299,6 +321,7 @@ class AngrObjectFactory(object):
                      extra_stop_points=extra_stop_points, thumb=thumb, backup_state=backup_state,
                      opt_level=opt_level, num_inst=num_inst, traceflags=traceflags,
                      strict_block_end=strict_block_end, collect_data_refs=collect_data_refs,
+                     cross_insn_opt=cross_insn_opt,
          )
 
     def fresh_block(self, addr, size, backup_state=None):
