@@ -1,17 +1,19 @@
 # pylint:disable=no-member
 import pickle
 import logging
+from typing import Optional, List, Dict, Tuple
 from collections import defaultdict
 
 import networkx
 
+from ...misc.ux import once
 from ...protos import cfg_pb2, primitives_pb2
 from ...serializable import Serializable
 from ...utils.enums_conv import cfg_jumpkind_to_pb, cfg_jumpkind_from_pb
 from ...errors import AngrCFGError
 from .cfg_node import CFGNode
 from .memory_data import MemoryData
-from ...misc.ux import once
+from .indirect_jump import IndirectJump
 
 
 l = logging.getLogger(name=__name__)
@@ -37,7 +39,7 @@ class CFGModel(Serializable):
         self.graph = networkx.DiGraph()
 
         # Jump tables
-        self.jump_tables = { }
+        self.jump_tables: Dict[int,IndirectJump] = { }
 
         # Memory references
         # A mapping between address and the actual data in memory
@@ -124,7 +126,7 @@ class CFGModel(Serializable):
         return cmsg
 
     @classmethod
-    def parse_from_cmessage(cls, cmsg, cfg_manager=None):  # pylint:disable=arguments-differ
+    def parse_from_cmessage(cls, cmsg, cfg_manager=None, loader=None):  # pylint:disable=arguments-differ
         if cfg_manager is None:
             # create a new model unassociated from any project
             model = cls(cmsg.ident)
@@ -158,6 +160,9 @@ class CFGModel(Serializable):
         # memory data
         for data_pb2 in cmsg.memory_data:
             md = MemoryData.parse_from_cmessage(data_pb2)
+            if loader is not None and md.content is None:
+                # fill in the content
+                md.fill_content(loader)
             model.memory_data[md.addr] = md
 
         return model
@@ -193,24 +198,25 @@ class CFGModel(Serializable):
             return self._nodes[block_id]
         return None
 
-    def get_any_node(self, addr, is_syscall=None, anyaddr=False, force_fastpath=False):
+    def get_any_node(self, addr: int, is_syscall: bool=None, anyaddr: bool=False,
+                     force_fastpath: bool=False) -> Optional[CFGNode]:
         """
         Get an arbitrary CFGNode (without considering their contexts) from our graph.
 
-        :param int addr:        Address of the beginning of the basic block. Set anyaddr to True to support arbitrary
+        :param addr:            Address of the beginning of the basic block. Set anyaddr to True to support arbitrary
                                 address.
-        :param bool is_syscall: Whether you want to get the syscall node or any other node. This is due to the fact that
+        :param is_syscall:      Whether you want to get the syscall node or any other node. This is due to the fact that
                                 syscall SimProcedures have the same address as the targer it returns to.
-                                None means get either, True means get a syscall node, False means get something that isn't
-                                a syscall node.
-        :param bool anyaddr:    If anyaddr is True, then addr doesn't have to be the beginning address of a basic
+                                None means get either, True means get a syscall node, False means get something that
+                                isn't a syscall node.
+        :param anyaddr:         If anyaddr is True, then addr doesn't have to be the beginning address of a basic
                                 block. By default the entire graph.nodes() will be iterated, and the first node
                                 containing the specific address is returned, which is slow. If you need to do many such
-                                queries, you may first call `generate_index()` to create some indices that may speed up the
-                                query.
-        :param bool force_fastpath: If force_fastpath is True, it will only perform a dict lookup in the _nodes_by_addr
-                                    dict.
-        :return: A CFGNode if there is any that satisfies given conditions, or None otherwise
+                                queries, you may first call `generate_index()` to create some indices that may speed up
+                                the query.
+        :param force_fastpath:  If force_fastpath is True, it will only perform a dict lookup in the _nodes_by_addr
+                                dict.
+        :return:                A CFGNode if there is any that satisfies given conditions, or None otherwise
         """
 
         # fastpath: directly look in the nodes list
@@ -253,7 +259,7 @@ class CFGModel(Serializable):
 
         return None
 
-    def get_all_nodes(self, addr, is_syscall=None, anyaddr=False):
+    def get_all_nodes(self, addr: int, is_syscall: bool=None, anyaddr: bool=False) -> List[CFGNode]:
         """
         Get all CFGNodes whose address is the specified one.
 
@@ -287,17 +293,17 @@ class CFGModel(Serializable):
 
         return self.graph.nodes()
 
-    def get_predecessors(self, cfgnode, excluding_fakeret=True, jumpkind=None):
+    def get_predecessors(self, cfgnode: CFGNode, excluding_fakeret: bool=True,
+                         jumpkind: Optional[str]=None) -> List[CFGNode]:
         """
         Get predecessors of a node in the control flow graph.
 
-        :param CFGNode cfgnode:             The node.
-        :param bool excluding_fakeret:      True if you want to exclude all predecessors that is connected to the node
-                                            with a fakeret edge.
-        :param str or None jumpkind:        Only return predecessors with the specified jumpkind. This argument will be
-                                            ignored if set to None.
-        :return:                            A list of predecessors
-        :rtype:                             list
+        :param cfgnode:             The node.
+        :param excluding_fakeret:   True if you want to exclude all predecessors that is connected to the node with a
+                                    fakeret edge.
+        :param jumpkind:            Only return predecessors with the specified jumpkind. This argument will be ignored
+                                    if set to None.
+        :return:                    A list of predecessors
         """
 
         if excluding_fakeret and jumpkind == 'Ijk_FakeRet':
@@ -358,7 +364,7 @@ class CFGModel(Serializable):
                 successors.append(suc)
         return successors
 
-    def get_successors_and_jumpkind(self, node, excluding_fakeret=True):
+    def get_successors_and_jumpkinds(self, node, excluding_fakeret=True):
         """
         Get a list of tuples where the first element is the successor of the CFG node and the second element is the
         jumpkind of the successor.
@@ -375,6 +381,26 @@ class CFGModel(Serializable):
                 successors.append((suc, data['jumpkind']))
         return successors
 
+    get_successors_and_jumpkind = get_successors_and_jumpkinds
+
+    def get_predecessors_and_jumpkinds(self, node: CFGNode, excluding_fakeret: bool=True) -> List[Tuple[CFGNode,str]]:
+        """
+        Get a list of tuples where the first element is the predecessor of the CFG node and the second element is the
+        jumpkind of the predecessor.
+
+        :param node:                The node.
+        :param excluding_fakeret:   True if you want to exclude all predecessors that are fall-through predecessors.
+        :return:                    A list of predecessors and their corresponding jumpkinds.
+        """
+
+        predecessors = []
+        for pred, _, data in self.graph.in_edges([node], data=True):
+            if not excluding_fakeret or data['jumpkind'] != 'Ijk_FakeRet':
+                predecessors.append((pred, data['jumpkind']))
+        return predecessors
+
+    get_predecessors_and_jumpkind = get_predecessors_and_jumpkinds
+
     def get_all_predecessors(self, cfgnode, depth_limit=None):
         """
         Get all predecessors of a specific node on the control flow graph.
