@@ -1,5 +1,6 @@
 import traceback
 import logging
+from typing import TYPE_CHECKING
 
 from archinfo.arch_soot import SootAddressDescriptor
 import archinfo
@@ -8,6 +9,10 @@ from ...codenode import BlockNode, HookNode, SyscallNode
 from ...engines.successors import SimSuccessors
 from ...serializable import Serializable
 from ...protos import cfg_pb2
+from ...errors import AngrError, SimError
+
+if TYPE_CHECKING:
+    from .cfg_model import CFGModel
 
 _l = logging.getLogger(__name__)
 
@@ -68,7 +73,7 @@ class CFGNode(Serializable):
         self.size = size
         self.simprocedure_name = simprocedure_name
         self.no_ret = no_ret
-        self._cfg_model = cfg
+        self._cfg_model: 'CFGModel' = cfg
         self.function_address = function_address
         self.block_id = block_id  # type: int or BlockID
         self.thumb = thumb
@@ -131,6 +136,12 @@ class CFGNode(Serializable):
     def predecessors(self):
         return self._cfg_model.get_predecessors(self)
 
+    def successors_and_jumpkinds(self, excluding_fakeret=True):
+        return self._cfg_model.get_successors_and_jumpkinds(self, excluding_fakeret=excluding_fakeret)
+
+    def predecessors_and_jumpkinds(self, excluding_fakeret=True):
+        return self._cfg_model.get_predecessors_and_jumpkinds(self, excluding_fakeret=excluding_fakeret)
+
     def get_data_references(self, kb=None):
         """
         Get the known data references for this CFGNode via the knowledge base.
@@ -204,6 +215,15 @@ class CFGNode(Serializable):
                   cfg=cfg,
                   block_id=block_id,
                   )
+        if cfg is not None:
+            # fill in self.instruction_addrs
+            proj = cfg.project
+            try:
+                obj.instruction_addrs = proj.factory.block(obj.addr, size=obj.size).instruction_addrs
+            except (AngrError, SimError):
+                # maybe this is a SimProcedure but not a block. ignore
+                # TODO: We should serialize information including is_simprocedure
+                pass
         return obj
 
     #
