@@ -4,21 +4,27 @@ import networkx
 import string
 import itertools
 from collections import defaultdict
-from typing import Union
+from typing import Union, Optional, Iterable
+from typing import Type # For some reasons the linter doesn't recognize the use in apply_definition but PyCharm needs it imported to correctly recognize it # pylint: disable=unused-import
 
 from cle.backends.symbol import Symbol
 from archinfo.arch_arm import get_real_address_if_arm
 import claripy
 
-from ...codenode import BlockNode, HookNode, SyscallNode
+from ...codenode import CodeNode, BlockNode, HookNode, SyscallNode
 from ...serializable import Serializable
 from ...errors import AngrValueError, SimEngineError, SimMemoryError
 from ...procedures import SIM_LIBRARIES
 from ...protos import function_pb2
+from ...calling_conventions import DEFAULT_CC
 from .function_parser import FunctionParser
 
 l = logging.getLogger(name=__name__)
 
+from ...sim_type import SimTypeFunction, parse_defns
+from ...calling_conventions import SimCC
+from ...project import Project
+
 
 class Function(Serializable):
     """
@@ -29,7 +35,7 @@ class Function(Serializable):
                  '_callout_sites', '_endpoints', '_call_sites', '_retout_sites', 'addr', '_function_manager',
                  'is_syscall', '_project', 'is_plt', 'addr', 'is_simprocedure', '_name', 'binary_name',
                  '_argument_registers', '_argument_stack_variables',
-                 'bp_on_stack', 'retaddr_on_stack', 'sp_delta', 'calling_convention', 'prototype', '_returning',
+                 'bp_on_stack', 'retaddr_on_stack', 'sp_delta', '_cc', '_prototype', '_returning',
                  'prepared_registers', 'prepared_stack_variables', 'registers_read_afterwards',
                  'startpoint', '_addr_to_block_node', '_block_sizes', '_block_cache', '_local_blocks',
                  '_local_block_addrs', 'info', 'tags', 'alignment',
@@ -84,12 +90,11 @@ class Function(Serializable):
         self.retaddr_on_stack = False
         self.sp_delta = 0
         # Calling convention
-        self.calling_convention = None
+        self._cc = None  # type: Optional[SimCC]
         # Function prototype
-        self.prototype = None
+        self._prototype = None  # type: Optional[SimTypeFunction]
         # Whether this function returns or not. `None` means it's not determined yet
         self._returning = None
-
         self.prepared_registers = set()
         self.prepared_stack_variables = set()
         self.registers_read_afterwards = set()
@@ -111,7 +116,7 @@ class Function(Serializable):
         # Stack offsets of those arguments passed in stack variables
         self._argument_stack_variables = []
 
-        self._project = None  # will be initialized upon the first access to self.project
+        self._project = None  # type: Optional[Project] # will be initialized upon the first access to self.project
 
         #
         # Initialize unspecified properties
@@ -174,7 +179,20 @@ class Function(Serializable):
         # Determine a calling convention
         # If it is a SimProcedure it might have a CC already defined which can be used
         if self.is_simprocedure and self.project is not None and self.addr in self.project._sim_procedures:
-            self.calling_convention = self.project._sim_procedures[self.addr].cc
+            simproc = self.project._sim_procedures[self.addr]
+            cc = simproc.cc
+            if cc is None:
+                arch = self.project.arch
+                if self.project.arch.name in DEFAULT_CC:
+                    cc = DEFAULT_CC[arch.name](arch)
+
+            # update cc.args according to num_args
+            # TODO: Handle non-traditional arguments like fp
+            if cc is not None and not cc.args and simproc.num_args:
+                args = cc.arg_locs(is_fp=[False] * simproc.num_args)  # arg_locs() uses cc.args
+                cc.args = args
+
+            self.calling_convention = cc
         else:
             self.calling_convention = None
 
@@ -192,7 +210,7 @@ class Function(Serializable):
         if self._project is None:
             # try to set it from function manager
             if self._function_manager is not None:
-                self._project = self._function_manager._kb._project
+                self._project = self._function_manager._kb._project #type: Optional[Project]
         return self._project
 
     @property
@@ -299,6 +317,67 @@ class Function(Serializable):
         # TODO: remove link register values
         return [const.value for block in self.blocks for const in block.vex.constants]
 
+    @property
+    def calling_convention(self):
+        """
+        Get the calling convention of this function.
+
+        :return:    The calling convention of this function.
+        :rtype:     Optional[SimCC]
+        """
+        return self._cc
+
+    @calling_convention.setter
+    def calling_convention(self, v):
+        """
+        Set the calling convention of this function. If the new cc has a function prototype, we will clear
+        self._prototype. Otherwise, if self.prototype is set, we will use it to update the function prototype of the new
+        cc, and then clear self._prototype. A warning message will be generated in either case.
+
+        :param Optional[SimCC] v:   The new calling convention.
+        :return:                    None
+        """
+        self._cc = v
+
+        if self._cc is not None:
+            if self._cc.func_ty is None and self._prototype is not None:
+                l.warning("The new calling convention for %r does not have a prototype associated. Using the existing "
+                          "function prototype to update the new calling convention. The existing function prototype "
+                          "will be removed.", self)
+                self._cc.func_ty = self._prototype
+                self._prototype = None
+            elif self._cc.func_ty is not None and self._prototype is not None:
+                l.warning("The new calling convention for %r already has a prototype associated. The existing function "
+                          "prototype will be removed.", self)
+                self._prototype = None
+
+    @property
+    def prototype(self):
+        """
+        Get the prototype of this function. We prioritize the function prototype that is set in self.calling_convention.
+
+        :return:    The function prototype.
+        :rtype:     Optional[SimTypeFunction]
+        """
+        if self._cc:
+            return self._cc.func_ty
+        else:
+            return self._prototype
+
+    @prototype.setter
+    def prototype(self, proto):
+        """
+        Set a new prototype to this function. If a calling convention is already set to this function, the new prototype
+        will be set to this calling convention instead.
+
+        :param Optional[SimTypeFunction] proto: The new prototype.
+        :return:    None
+        """
+        if self._cc:
+            self._cc.func_ty = proto.with_arch(self.project.arch) if proto else None
+        else:
+            self._prototype = proto
+
     @classmethod
     def _get_cmsg(cls):
         return function_pb2.Function()
@@ -307,13 +386,13 @@ class Function(Serializable):
         return FunctionParser.serialize(self)
 
     @classmethod
-    def parse_from_cmessage(cls, cmsg, function_manager=None):  # pylint:disable=arguments-differ
+    def parse_from_cmessage(cls, cmsg, **kwargs):
         """
         :param cmsg:
 
         :return Function: The function instantiated out of the cmsg data.
         """
-        return FunctionParser.parse_from_cmsg(cmsg, function_manager)
+        return FunctionParser.parse_from_cmsg(cmsg, **kwargs)
 
 
     def string_references(self, minimum_length=2, vex_only=False):
@@ -497,8 +576,9 @@ class Function(Serializable):
 
     def __repr__(self):
         if self.is_syscall:
-            return '<Syscall function %s (%s)>' % (self.name, self.addr)
-        return '<Function %s (%s)>' % (self.name, self.addr)
+            return '<Syscall function %s (%s)>' % (self.name,
+                                                   hex(self.addr) if isinstance(self.addr, int) else self.addr)
+        return '<Function %s (%s)>' % (self.name, hex(self.addr) if isinstance(self.addr, int) else self.addr)
 
     @property
     def endpoints(self):
@@ -679,7 +759,7 @@ class Function(Serializable):
 
         self.transition_graph[src][dst]['confirmed'] = True
 
-    def _transit_to(self, from_node, to_node, outside=False, ins_addr=None, stmt_idx=None):
+    def _transit_to(self, from_node, to_node, outside=False, ins_addr=None, stmt_idx=None, is_exception=False):
         """
         Registers an edge between basic blocks in this function's transition graph.
         Arguments are CodeNode objects.
@@ -704,14 +784,15 @@ class Function(Serializable):
             else:
                 self._register_nodes(True, from_node)
 
+        type_ = 'transition' if not is_exception else 'exception'
         if to_node is not None:
-            self.transition_graph.add_edge(from_node, to_node, type='transition', outside=outside, ins_addr=ins_addr,
+            self.transition_graph.add_edge(from_node, to_node, type=type_, outside=outside, ins_addr=ins_addr,
                                            stmt_idx=stmt_idx
                                            )
 
         if outside:
             # this node is an endpoint of the current function
-            self._add_endpoint(from_node, 'transition')
+            self._add_endpoint(from_node, type_)
 
         # clear the cache
         self._local_transition_graph = None
@@ -764,7 +845,7 @@ class Function(Serializable):
         self._local_transition_graph = None
 
     def _return_from_call(self, from_func, to_node, to_outside=False):
-        self.transition_graph.add_edge(from_func, to_node, type='real_return', to_outside=to_outside)
+        self.transition_graph.add_edge(from_func, to_node, type='return', to_outside=to_outside)
         for _, _, data in self.transition_graph.in_edges(to_node, data=True):
             if 'type' in data and data['type'] == 'fake_return':
                 data['confirmed'] = True
@@ -777,6 +858,8 @@ class Function(Serializable):
 
         for node in nodes:
             self.transition_graph.add_node(node)
+            if not isinstance(node, CodeNode):
+                continue
             node._graph = self.transition_graph
             if node.addr not in self or self._block_sizes[node.addr] == 0:
                 self._block_sizes[node.addr] = node.size
@@ -890,11 +973,11 @@ class Function(Serializable):
                         self._callout_sites.add(the_node)
                         self._add_endpoint(the_node, 'call')
 
-    def get_call_sites(self):
+    def get_call_sites(self) -> Iterable[int]:
         """
         Gets a list of all the basic blocks that end in calls.
 
-        :return:                    A list of the addresses of the blocks that end in calls.
+        :return:                    A view of the addresses of the blocks that end in calls.
         """
         return self._call_sites.keys()
 
@@ -925,7 +1008,14 @@ class Function(Serializable):
     @property
     def graph(self):
         """
-        :return networkx.DiGraph: A local transition graph that only contain nodes in current function.
+        Get a local transition graph. A local transition graph is a transition graph that only contains nodes that
+        belong to the current function. All edges, except for the edges going out from the current function or coming
+        from outside the current function, are included.
+
+        The generated graph is cached in self._local_transition_graph.
+
+        :return:    A local transition graph.
+        :rtype:     networkx.DiGraph
         """
 
         if self._local_transition_graph is not None:
@@ -938,7 +1028,7 @@ class Function(Serializable):
             g.add_node(block)
         for src, dst, data in self.transition_graph.edges(data=True):
             if 'type' in data:
-                if data['type']  == 'transition' and ('outside' not in data or data['outside'] is False):
+                if data['type'] in ('transition', 'exception') and ('outside' not in data or data['outside'] is False):
                     g.add_edge(src, dst, **data)
                 elif data['type'] == 'fake_return' and 'confirmed' in data and \
                         ('outside' not in data or data['outside'] is False):
@@ -948,6 +1038,93 @@ class Function(Serializable):
 
         return g
 
+    def graph_ex(self, exception_edges=True):
+        """
+        Get a local transition graph with a custom configuration. A local transition graph is a transition graph that
+        only contains nodes that belong to the current function. This method allows user to exclude certain types of
+        edges together with the nodes that are only reachable through such edges, such as exception edges.
+
+        The generated graph is not cached.
+
+        :param bool exception_edges:    Should exception edges and the nodes that are only reachable through exception
+                                        edges be kept.
+        :return:                        A local transition graph with a special configuration.
+        :rtype:                         networkx.DiGraph
+        """
+
+        # graph_ex() should not impact any already cached graph
+        old_cached_graph = self._local_transition_graph
+        graph = self.graph
+        self._local_transition_graph = old_cached_graph  # restore the cached graph
+
+        # fast path
+        if exception_edges:
+            return graph
+
+        # BFS on local graph but ignoring certain types of graphs
+        g = networkx.DiGraph()
+        queue = [ n for n in graph if n is self.startpoint or graph.in_degree[n] == 0 ]
+        traversed = set(queue)
+
+        while queue:
+            node = queue.pop(0)
+
+            g.add_node(node)
+            for _, dst, edge_data in graph.out_edges(node, data=True):
+                edge_type = edge_data.get('type', None)
+                if not exception_edges and edge_type == 'exception':
+                    # ignore this edge
+                    continue
+                g.add_edge(node, dst, **edge_data)
+
+                if dst not in traversed:
+                    traversed.add(dst)
+                    queue.append(dst)
+
+        return g
+
+    def transition_graph_ex(self, exception_edges=True):
+        """
+        Get a transition graph with a custom configuration. This method allows user to exclude certain types of edges
+        together with the nodes that are only reachable through such edges, such as exception edges.
+
+        The generated graph is not cached.
+
+        :param bool exception_edges:    Should exception edges and the nodes that are only reachable through exception
+                                        edges be kept.
+        :return:                        A local transition graph with a special configuration.
+        :rtype:                         networkx.DiGraph
+        """
+
+        graph = self.transition_graph
+
+        # fast path
+        if exception_edges:
+            return graph
+
+        # BFS on local graph but ignoring certain types of graphs
+        g = networkx.DiGraph()
+        queue = [ n for n in graph if n is self.startpoint or graph.in_degree[n] == 0 ]
+        traversed = set(queue)
+
+        while queue:
+            node = queue.pop(0)
+            traversed.add(node)
+
+            g.add_node(node)
+            for _, dst, edge_data in graph.out_edges(node, data=True):
+                edge_type = edge_data.get('type', None)
+                if not exception_edges and edge_type == 'exception':
+                    # ignore this edge
+                    continue
+                g.add_edge(node, dst, **edge_data)
+
+                if dst not in traversed:
+                    traversed.add(dst)
+                    queue.append(dst)
+
+        return g
+
     def subgraph(self, ins_addrs):
         """
         Generate a sub control flow graph of instruction addresses based on self.graph
@@ -1251,8 +1428,7 @@ class Function(Serializable):
         if not library.has_prototype(self.name):
             return
 
-        proto = library.prototypes[self.name]
-
+        proto = library.get_prototype(self.name)
         self.prototype = proto
         if self.calling_convention is not None:
             self.calling_convention.args = None
@@ -1269,15 +1445,48 @@ class Function(Serializable):
 
     @property
     def demangled_name(self):
-
         """
         if self.name[0:2] == "_Z":
-            ast = parse(self.name)
+            try:
+                ast = parse(self.name)
+            except NotImplementedError:
+                return self.name
             if ast:
                 return ast.__str__()
         """
         return self.name
 
+    def apply_definition(self, definition, calling_convention=None):
+        """
+
+        :param str definition:
+        :param Optional[Union[SimCC, Type[SimCC]]] calling_convention:
+        :return None:
+        """
+        if not definition.endswith(";"):
+            definition += ";"
+        func_def = parse_defns(definition)
+        if len(func_def.keys()) > 1:
+            raise Exception("Too many definitions: %s " % list(func_def.keys()))
+
+        name, ty = func_def.popitem() # type: str, SimTypeFunction
+        self.name = name
+        # setup the calling convention
+        # If a SimCC object is passed assume that this is sane and just use it
+        if isinstance(calling_convention, SimCC):
+            self.calling_convention = calling_convention
+
+        # If it is a subclass of SimCC we can instantiate it
+        elif isinstance(calling_convention, type) and issubclass(calling_convention, SimCC):
+            self.calling_convention = calling_convention(self.project.arch, func_ty=ty)
+
+        # If none is specified default to something
+        elif calling_convention is None:
+            self.calling_convention = self.project.factory.cc(func_ty=ty)
+
+        else:
+            raise TypeError("calling_convention has to be one of: [SimCC, type(SimCC), None]")
+
     def copy(self):
         func = Function(self._function_manager, self.addr, name=self.name, syscall=self.is_syscall)
         func.transition_graph = networkx.DiGraph(self.transition_graph)
