@@ -5,16 +5,18 @@ from io import BytesIO, IOBase
 import pickle
 import string
 from collections import defaultdict
+from typing import Dict, Any
 
 import archinfo
 from archinfo.arch_soot import SootAddressDescriptor, ArchSoot
 import cle
 
 from .misc.ux import deprecated
+from .errors import AngrNoPluginError
 
 l = logging.getLogger(name=__name__)
 
-def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False):
+def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False, **kwargs):
     """
     Load a new project based on a snippet of assembly or bytecode.
 
@@ -38,7 +40,8 @@ def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False)
                 'arch': arch,
                 'entry_point': start_offset,
                 'base_addr': load_address,
-            }
+            },
+        **kwargs
         )
 
 
@@ -94,7 +97,7 @@ class Project:
                  exclude_sim_procedures_list=(),
                  arch=None, simos=None,
                  engine=None,
-                 load_options=None,
+                 load_options: Dict[str, Any]=None,
                  translation_cache=True,
                  support_selfmodifying_code=False,
                  store_function=None,
@@ -131,12 +134,11 @@ class Project:
         if isinstance(arch, str):
             self.arch = archinfo.arch_from_id(arch)  # may raise ArchError, let the user see this
         elif isinstance(arch, archinfo.Arch):
-            self.arch = arch
+            self.arch = arch # type: archinfo.Arch
         elif arch is None:
             self.arch = self.loader.main_object.arch
         else:
             raise ValueError("Invalid arch specification.")
-
         # Step 3: Set some defaults and set the public and private properties
         if not default_analysis_mode:
             default_analysis_mode = 'symbolic'
@@ -187,11 +189,12 @@ class Project:
         self.factory = AngrObjectFactory(self, default_engine=engine)
 
         # Step 4.2: Analyses
-        self.analyses = AnalysesHub(self)
-        self.analyses.use_plugin_preset(analyses_preset if analyses_preset is not None else 'default')
+        self._analyses_preset = analyses_preset
+        self.analyses = None
+        self._initialize_analyses_hub()
 
         # Step 4.3: ...etc
-        self.kb = KnowledgeBase(self)
+        self.kb = KnowledgeBase(self, name="global")
 
         # Step 5: determine the guest OS
         if isinstance(simos, type) and issubclass(simos, SimOS):
@@ -220,6 +223,13 @@ class Project:
         # Step 7: Run OS-specific configuration
         self.simos.configure_project()
 
+    def _initialize_analyses_hub(self):
+        """
+        Initializes self.analyses using a given preset.
+        """
+        self.analyses = AnalysesHub(self)
+        self.analyses.use_plugin_preset(self._analyses_preset if self._analyses_preset is not None else 'default')
+
     def _register_object(self, obj, sim_proc_arch):
         """
         This scans through an objects imports and hooks them with simprocedures from our library whenever possible
@@ -330,7 +340,16 @@ class Project:
                 else:
                     if not func.is_weak:
                         l.info("Using stub SimProcedure for unresolved %s", export.name)
-                        self.hook_symbol(export.rebased_addr, missing_libs[0].get(export.name, sim_proc_arch))
+                        the_lib = missing_libs[0]
+                        if export.name and export.name.startswith("_Z"):
+                            # GNU C++ name. Use a C++ library to create the stub
+                            if 'libstdc++.so' in SIM_LIBRARIES:
+                                the_lib = SIM_LIBRARIES['libstdc++.so']
+                            else:
+                                l.critical("Does not find any C++ library in SIM_LIBRARIES. We may not correctly "
+                                           "create the stub or resolve the function prototype for name %s.", export.name)
+
+                        self.hook_symbol(export.rebased_addr, the_lib.get(export.name, sim_proc_arch))
 
             # Step 2.5: If 2.4 didn't work (we have NO SimLibraries to work with), just
             # use the vanilla ReturnUnconstrained, assuming that this isn't a weak func
@@ -554,7 +573,7 @@ class Project:
         self.unhook(hook_addr)
         return True
 
-    def rehook_symbol(self, new_address, symbol_name):
+    def rehook_symbol(self, new_address, symbol_name, stubs_on_sync):
         """
         Move the hook for a symbol to a specific address
         :param new_address: the new address that will trigger the SimProc execution
@@ -563,6 +582,11 @@ class Project:
         """
         new_sim_procedures = {}
         for key_address, simproc_obj in self._sim_procedures.items():
+
+            # if we don't want stubs during the sync let's skip those, we will execute the real function.
+            if not stubs_on_sync and simproc_obj.is_stub:
+                continue
+
             if simproc_obj.display_name == symbol_name:
                 new_sim_procedures[new_address] = simproc_obj
             else:
@@ -634,12 +658,21 @@ class Project:
         try:
             store_func, load_func = self.store_function, self.load_function
             self.store_function, self.load_function = None, None
-            return dict(self.__dict__)
+            # ignore analyses. we re-initialize analyses when restoring from pickling so that we do not lose any newly
+            # added analyses classes
+            d = dict((k, v) for k, v in self.__dict__.items() if k not in {'analyses', })
+            return d
         finally:
             self.store_function, self.load_function = store_func, load_func
 
     def __setstate__(self, s):
         self.__dict__.update(s)
+        try:
+            self._initialize_analyses_hub()
+        except AngrNoPluginError:
+            l.warning("Plugin preset %s does not exist any more. Fall back to the default preset.")
+            self._analyses_preset = 'default'
+            self._initialize_analyses_hub()
 
     def _store(self, container):
         # If container is a filename.
