@@ -104,10 +104,6 @@ class SimProcedure:
         self.abi = None
         self.symbolic_return = symbolic_return
 
-        # types
-        self.argument_types = { } # a dictionary of index-to-type (i.e., type of arg 0: SimTypeString())
-        self.return_type = None
-
         # set some properties about the type of procedure this is
         self.returns = returns if returns is not None else not self.NO_RET
         self.is_syscall = is_syscall
@@ -266,6 +262,7 @@ class SimProcedure:
     ADDS_EXITS = False
     IS_FUNCTION = True
     ARGS_MISMATCH = False
+    ALT_NAMES = None  # alternative names
     local_vars = ()
 
     def run(self, *args, **kwargs): # pylint: disable=unused-argument
@@ -286,9 +283,9 @@ class SimProcedure:
 
         if self.ADDS_EXITS:
             raise SimProcedureError("static_exits() is not implemented for %s" % self)
-        else:
-            # This SimProcedure does not add any new exit
-            return [ ]
+
+        # This SimProcedure does not add any new exit
+        return []
 
     #
     # misc properties
@@ -492,6 +489,22 @@ class SimProcedure:
     def _prepare_ret_state(self):
         pass
 
+    @property
+    def argument_types(self):  # pylint: disable=no-self-use
+        return None
+
+    @argument_types.setter
+    def argument_types(self, v):  # pylint: disable=unused-argument,no-self-use
+        l.critical("SimProcedure.argument_types is deprecated. specify the function signature in the cc")
+
+    @property
+    def return_type(self):  # pylint: disable=no-self-use
+        return None
+
+    @return_type.setter
+    def return_type(self, v):  # pylint: disable=unused-argument,no-self-use
+        l.critical("SimProcedure.return_type is deprecated. specify the function signature in the cc")
+
 
 from . import sim_options as o
 from angr.errors import SimProcedureError, SimProcedureArgumentError, SimShadowStackError
